#include "include.h"

#define MAX_LEVEL 9
char level_name[MAX_LEVEL][80] = {
	"E1M1",
	"E1M2",
	"E1M3",
	"E1M4",
	"E1M5",
	"E1M6",
	"E1M7",
	"E1M8",
	"E1M9",
};


typedef struct
{
	float x;
	float y;
	float scale;
} offset_t;

offset_t level_offsets[MAX_LEVEL] = {
	{1180, 4880, 2.5f},
	{4700, 1460, 1.5f},
	{4720, 3740, 2.0f},
	{2680,  640, 2.5},
	{3780, 920, 2.0f},
	{4780, 2420, 1.5},
	{4140, 2840, 2.0f},
	{6360, 520, 1.0f},
	{3220, 1600, 2.0}
};


char *get_file(char *filename, unsigned int *size)
{
	FILE	*file;
	char	*buffer;
	int	file_size, bytes_read;

	file = fopen(filename, "rb");
	if (file == NULL)
		return 0;
	fseek(file, 0, SEEK_END);
	file_size = ftell(file);
	fseek(file, 0, SEEK_SET);
	buffer = new char[file_size + 1];
	bytes_read = (int)fread(buffer, sizeof(char), file_size, file);
	if (bytes_read != file_size)
	{
		delete[] buffer;
		fclose(file);
		return 0;
	}
	fclose(file);
	buffer[file_size] = '\0';

	if (size != NULL)
	{
		*size = file_size;
	}

	return buffer;
}



void load_wad(waddata_t *data, int level)
{
	int lump_size;
	char *pdata = NULL;

	data->pvert = (vert_t *)get_wadfile("DOOM1.WAD", level_name[level], "VERTEXES", &lump_size, &pdata);
	data->num_vert = lump_size / (sizeof(vert_t));

	data->pline = (linedef_t *)get_wadfile("DOOM1.WAD", level_name[level], "LINEDEFS", &lump_size, &pdata);
	data->num_line = lump_size / (sizeof(linedef_t));

	data->pthing = (thing_t *)get_wadfile("DOOM1.WAD", level_name[level], "THINGS", &lump_size, &pdata);
	data->num_thing = lump_size / (sizeof(thing_t));

	data->pside = (sidedef_t *)get_wadfile("DOOM1.WAD", level_name[level], "SIDEDEFS", &lump_size, &pdata);
	data->num_side = lump_size / (sizeof(sidedef_t));

	data->psector = (sector_t *)get_wadfile("DOOM1.WAD", level_name[level], "SECTORS", &lump_size, &pdata);
	data->num_sector = lump_size / (sizeof(sector_t));

	data->pseg = (seg_t *)get_wadfile("DOOM1.WAD", level_name[level], "SEGS", &lump_size, &pdata);
	data->num_seg = lump_size / (sizeof(seg_t));

	data->psubsector = (subsector_t *)get_wadfile("DOOM1.WAD", level_name[level], "SSECTORS", &lump_size, &pdata);
	data->num_subsector = lump_size / (sizeof(subsector_t));

	data->pnode = (node_t *)get_wadfile("DOOM1.WAD", level_name[level], "NODES", &lump_size, &pdata);
	data->num_node = lump_size / (sizeof(node_t));
}


LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	PSTR szCmdLine, int iCmdShow)
{
	static TCHAR szAppName[] = TEXT("DoomView");
	HWND         hwnd;
	MSG          msg;
	WNDCLASS     wndclass;

	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = WndProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wndclass.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
	wndclass.lpszMenuName = NULL;
	wndclass.lpszClassName = szAppName;

	if (!RegisterClass(&wndclass))
	{
		MessageBox(NULL, TEXT("Program requires Windows NT!"),
			szAppName, MB_ICONERROR);
		return 0;
	}

	hwnd = CreateWindow(szAppName, TEXT("DoomView"),
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT,
		CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInstance, NULL);

	ShowWindow(hwnd, iCmdShow);
	UpdateWindow(hwnd);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static int  cxClient, cyClient;
	HDC         hdc;
	int         i, j;
	PAINTSTRUCT ps;
	static float scale = 2.5f;
	static float offset_x = 1180.0f;
	static float offset_y = 2020.0f;
	static int level = 0;
	static char info[512];
	static waddata_t data;
	static HPEN red;
	static HPEN blue;

	switch (message)
	{
	case WM_CREATE:
		 load_wad(&data, level);

		 scale = level_offsets[level].scale;
		 offset_x = level_offsets[level].x;
		 offset_y = level_offsets[level].y;
		
		 red = CreatePen(PS_SOLID, 0, RGB(255, 0, 0));
		 blue = CreatePen(PS_SOLID, 0, RGB(0, 0, 255));
		 return 0;
	case WM_SIZE:
		cxClient = LOWORD(lParam);
		cyClient = HIWORD(lParam);
		return 0;

	case WM_KEYDOWN:
		switch (wParam)
		{
		case VK_NEXT:
			scale += 0.5f;
			break;
		case VK_PRIOR:
			scale -= 0.5f;
			break;
		case VK_LEFT:
			offset_x -= 20.0f;
			break;
		case VK_RIGHT:
			offset_x += 20.0f;
			break;
		case VK_DOWN:
			offset_y -= 20.0f;
			break;
		case VK_UP:
			offset_y += 20.0f;
			break;
		case VK_ADD:
			level++;
			if (level >= MAX_LEVEL)
				level = MAX_LEVEL - 1;

			load_wad(&data, level);
			scale = level_offsets[level].scale;
			offset_x = level_offsets[level].x;
			offset_y = level_offsets[level].y;

			break;
		case VK_SUBTRACT:
			level--;
			if (level < 0)
				level = 0;

			load_wad(&data, level);
			scale = level_offsets[level].scale;
			offset_x = level_offsets[level].x;
			offset_y = level_offsets[level].y;

			break;

		}

		InvalidateRect(hwnd, NULL, true);

		return 0;

	case WM_PAINT:
		hdc = BeginPaint(hwnd, &ps);

		RECT rect;

		rect.left = 0;
		rect.top = 0;
		rect.bottom = 50;
		rect.right = cxClient;

		sprintf(info, "%s scale %f offsetx %f offsety %f", level_name[level], scale, offset_x, offset_y);
		DrawText(hdc, info, -1, &rect, 0);


		// draw vertices
		SelectObject(hdc, GetStockObject(BLACK_PEN));
		for (i = 0; i < data.num_vert; i += 2)
		{
			float x = (data.pvert[i].x * scale + offset_x * scale) / 10.0f;
			float y = (data.pvert[i].y * scale + offset_y * scale) / 10.0f;

			Rectangle(hdc,
				(x - 3.0f),
				(y - 3.0f),
				(x + 3.0f),
				(y + 3.0f));
		}

		// draw lines
		for (i = 0; i < data.num_line; i++)
		{
			int a = data.pline[i].start_vert;
			int b = data.pline[i].end_vert;

			float startx = (data.pvert[a].x * scale + offset_x * scale) / 10.0f;
			float starty = (data.pvert[a].y * scale + offset_y * scale) / 10.0f;
			float endx = (data.pvert[b].x * scale + offset_x * scale) / 10.0f;
			float endy = (data.pvert[b].y * scale + offset_y * scale) / 10.0f;

			MoveToEx(hdc, startx, starty, NULL);
			LineTo(hdc, endx, endy);
		}


		// draw things
		SelectObject(hdc, red);
		for (i = 0; i < data.num_thing; i++)
		{
			float x = (data.pthing[i].x * scale + offset_x * scale) / 10.0f;
			float y = (data.pthing[i].y * scale + offset_y * scale) / 10.0f;

			Rectangle(hdc,
				(x - 3.0f),
				(y - 3.0f),
				(x + 3.0f),
				(y + 3.0f));
		}

		// draw subsectors
		/*
		SelectObject(hdc, blue);
		for (i = 0; i < data.num_subsector; i++)
		{
			int first = data.psubsector[i].first;
			int count = data.psubsector[i].count;

			for (j = first; j < first + count; j++)
			{
				int a = data.pseg[j].start_vert;
				int b = data.pseg[j].end_vert;

				float startx = (data.pvert[a].x * scale + offset_x * scale) / 10.0f;
				float starty = (data.pvert[a].y * scale + offset_y * scale) / 10.0f;
				float endx = (data.pvert[b].x * scale + offset_x * scale) / 10.0f;
				float endy = (data.pvert[b].y * scale + offset_y * scale) / 10.0f;

				MoveToEx(hdc, startx, starty, NULL);
				LineTo(hdc, endx, endy);
			}
		}
*/

		EndPaint(hwnd, &ps);
		return 0;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hwnd, message, wParam, lParam);
}
